dnl ##################################################################
dnl The macro AC_UNP_CHECK_TYPE
dnl comes from aclocal.m4 in the sample code associated
dnl with "UNIX Network Programming: Volume 1", Second Edition,
dnl by W. Richard Stevens.  That sample code is available via
dnl ftp://ftp.kohala.com/pub/rstevens/unpv12e.tar.gz.
dnl
dnl Version $Id: aclocal.m4,v 1.4 2008/10/17 20:09:12 root Exp $
dnl
dnl Usage:
dnl   AC_UNP_CHECK_TYPE(type, default)
dnl Like the standard AC_CHECK_TYPE macro, this checks if the type 'type'
dnl is defined in certain system headers, and if not, defines 'type' to
dnl be the C builtin type 'default'.
dnl
dnl You would use this macro instead of the standard AC_CHECK_TYPE
dnl because AC_CHECK_TYPE #includes only <sys/types.h>, <stdlib.h>,
dnl and <stddef.h>.
dnl Unfortunately, many implementations today hide typedefs in weird
dnl locations: Solaris 2.5.1 has uint8_t and uint32_t in <pthread.h>.
dnl SunOS 4.1.x has int8_t in <sys/bitypes.h>.
dnl So AC_UNP_CHECK_TYPE #includes more headers, then tries to compile
dnl a test program that uses the typedef.
dnl
dnl This macro should be invoked in configure.in after all the header checks
dnl (e.g. AC_CHECK_HEADERS) have been performed, since our test program will
dnl #include "confdefs.h" (the C macro definitions accumulated by configure so far),
dnl then will #include various header files based on some HAVE_foo_H values we expect
dnl may appear in "confdefs.h".  Of course, to make this work, you should have invoked
dnl AC_CHECK_HEADERS in configure.in on all those possible header files.
dnl
dnl If a type you need to check for might be defined in some *other* header file, you'll
dnl need to edit your AC_CHECK_HEADERS invocations in configure.in to add that header
dnl file, then edit the macro definition below to #include that header file
dnl (based on the appropriate HAVE_foo_H value).
dnl
AC_DEFUN([AC_UNP_CHECK_TYPE],
	[AC_MSG_CHECKING(if $1 defined)
	AC_CACHE_VAL(ac_cv_type_$1,
		AC_COMPILE_IFELSE( [AC_LANG_SOURCE(
							[[
								#include	"confdefs.h"	/* the header built by configure so far */
								#ifdef	HAVE_SYS_TYPES_H
								#  include	<sys/types.h>
								#endif
								#ifdef	HAVE_SYS_SOCKET_H
								#  include	<sys/socket.h>
								#endif
								#ifdef	HAVE_SYS_TIME_H
								#  include    <sys/time.h>
								#endif
								#ifdef	HAVE_NETINET_IN_H
								#  include    <netinet/in.h>
								#endif
								#ifdef	HAVE_ARPA_INET_H
								#  include    <arpa/inet.h>
								#endif
								#ifdef	HAVE_ERRNO_H
								#  include    <errno.h>
								#endif
								#ifdef	HAVE_FCNTL_H
								#  include    <fcntl.h>
								#endif
								#ifdef	HAVE_NETDB_H
								#  include	<netdb.h>
								#endif
								#ifdef	HAVE_SIGNAL_H
								#  include	<signal.h>
								#endif
								#ifdef	HAVE_STDIO_H
								#  include	<stdio.h>
								#endif
								#ifdef	HAVE_STDLIB_H
								#  include	<stdlib.h>
								#endif
								#ifdef	HAVE_STRING_H
								#  include	<string.h>
								#endif
								#ifdef	HAVE_SYS_STAT_H
								#  include	<sys/stat.h>
								#endif
								#ifdef	HAVE_SYS_UIO_H
								#  include	<sys/uio.h>
								#endif
								#ifdef	HAVE_UNISTD_H
								#  include	<unistd.h>
								#endif
								#ifdef	HAVE_SYS_WAIT_H
								#  include	<sys/wait.h>
								#endif
								#ifdef	HAVE_SYS_UN_H
								#  include	<sys/un.h>
								#endif
								#ifdef	HAVE_SYS_SELECT_H
								# include   <sys/select.h>
								#endif
								#ifdef	HAVE_STRINGS_H
								# include   <strings.h>
								#endif
								#ifdef	HAVE_SYS_IOCTL_H
								# include   <sys/ioctl.h>
								#endif
								#ifdef	HAVE_SYS_FILIO_H
								# include   <sys/filio.h>
								#endif
								#ifdef	HAVE_SYS_SOCKIO_H
								# include   <sys/sockio.h>
								#endif
								#ifdef	HAVE_PTHREAD_H
								#  include	<pthread.h>
								#endif
							]]
							[[
			 					$1 foo ;
							]]
							)
						],
			ac_cv_type_$1=yes
		,
			ac_cv_type_$1=no
		)
	)
		AC_MSG_RESULT($ac_cv_type_$1)
		if test $ac_cv_type_$1 = no ; then
			AC_DEFINE($1, $2, [On systems that lack a typedef for $1, define it as $2.])
		fi
])

dnl ##################################################################
dnl The macro AC_CHECK_STRUCT_FOR is 
dnl Available from the GNU Autoconf Macro Archive at:
dnl http://www.gnu.org/software/ac-archive/htmldoc/ac_check_struct_for.html
dnl See the COPYING file for details.
dnl
dnl @synopsis AC_CHECK_STRUCT_FOR(INCLUDES,STRUCT,MEMBER,DEFINE,[no])
dnl
dnl Checks STRUCT for MEMBER and defines DEFINE if found.
dnl
dnl @version $Id: aclocal.m4,v 1.4 2008/10/17 20:09:12 root Exp $
dnl @author Wes Hardaker <wjhardaker@ucdavis.edu>
dnl
AC_DEFUN([AC_CHECK_STRUCT_FOR],[
ac_safe_struct=`echo "$2" | sed 'y%./+-%__p_%'`
ac_safe_member=`echo "$3" | sed 'y%./+-%__p_%'`
ac_safe_all="ac_cv_struct_${ac_safe_struct}_has_${ac_safe_member}"
changequote(, )dnl
  ac_uc_define=STRUCT_`echo "${ac_safe_struct}_HAS_${ac_safe_member}" | sed 'y%abcdefghijklmnopqrstuvwxyz./-%ABCDEFGHIJKLMNOPQRSTUVWXYZ___%'`
changequote([, ])dnl

AC_MSG_CHECKING([for $2.$3])
AC_CACHE_VAL($ac_safe_all,
[
if test "x$4" = "x"; then
  defineit="= 0"
elif test "x$4" = "xno"; then
  defineit=""
else
  defineit="$4"
fi
AC_TRY_COMPILE([
$1
],[
struct $2 testit;
testit.$3 $defineit;
], eval "${ac_safe_all}=yes", eval "${ac_safe_all}=no" )
])

if eval "test \"x$`echo ${ac_safe_all}`\" = \"xyes\""; then
  AC_MSG_RESULT(yes)
  AC_DEFINE_UNQUOTED($ac_uc_define)
else
  AC_MSG_RESULT(no)
fi
])

dnl ##################################################################
dnl The macro AC_SYS_NERR
dnl is based on code from configure.in distributed with LPRng-2.0.8.
dnl See the COPYING file for details.
dnl
dnl Version $Id: aclocal.m4,v 1.4 2008/10/17 20:09:12 root Exp $
dnl 
dnl Usage:
dnl  AC_SYS_NERR
dnl Check if sys_nerr exists.
dnl Tries to link a program that mentions sys_nerr.
dnl If it exists, defines C preprocessor macro HAVE_SYS_NERR.
dnl Sets the cache variable ac_cv_var_sys_nerr to 'yes' or 'no'.
dnl
AC_DEFUN([AC_SYS_NERR],
[AC_MSG_CHECKING(for sys_nerr)
AC_CACHE_VAL(ac_cv_var_sys_nerr,
[AC_TRY_LINK(,[extern int sys_nerr; return (sys_nerr);],
	ac_cv_var_sys_nerr=yes, ac_cv_var_sys_nerr=no)
])
AC_MSG_RESULT($ac_cv_var_sys_nerr)
if test $ac_cv_var_sys_nerr = yes; then
	AC_DEFINE(HAVE_SYS_NERR, [], [Define the following if sys_nerr exists])
fi;
])

dnl ##################################################################
dnl The macro AC_DECL_SYS_NERR
dnl is based on code from configure.in distributed with LPRng-2.0.8.
dnl See the COPYING file for details.
dnl
dnl Usage:
dnl  AC_DECL_SYS_NERR
dnl Check if sys_nerr is declared in one of the usual system headers.
dnl Tries to compile a program that uses sys_nerr.
dnl If the declaration exists, defines C preprocessor macro HAVE_SYS_NERR_DECL.
dnl Sets the cache variable ac_cv_decl_sys_nerr to 'yes' or 'no'.
dnl
dnl This macro should be invoked in configure.in after all the header checks
dnl (e.g. AC_CHECK_HEADERS) have been performed, since our test program will
dnl #include "confdefs.h" (the C macro definitions accumulated by configure so far),
dnl then will #include various header files based on HAVE_STDLIB_H and HAVE_UNISTD_H we expect
dnl may appear in "confdefs.h".  Of course, to make this work, you should have invoked
dnl AC_CHECK_HEADERS(stdlib.h unistd.h) in configure.in.
dnl
AC_DEFUN([AC_DECL_SYS_NERR],
[AC_MSG_CHECKING(for sys_nerr declaration)
AC_CACHE_VAL(ac_cv_decl_sys_nerr,
[AC_TRY_COMPILE([
#include "confdefs.h"    /* the header built by configure so far */
#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif],
[printf("%d",sys_nerr);],
ac_cv_decl_sys_nerr=yes, ac_cv_decl_sys_nerr=no)
])
AC_MSG_RESULT($ac_cv_decl_sys_nerr)
if test $ac_cv_decl_sys_nerr = yes; then
    AC_DEFINE(HAVE_SYS_NERR_DECL, [], [Define the following if sys_nerr is declared])
fi
])

dnl ##################################################################
dnl The macro AC_SYS_ERRLIST
dnl is based on code from configure.in distributed with LPRng-2.0.8.
dnl See the COPYING file for details.
dnl
dnl Usage:
dnl  AC_SYS_ERRLIST
dnl Check if sys_errlist exists.
dnl Tries to link a program that mentions sys_errlist.
dnl If it exists, defines C preprocessor macro HAVE_SYS_ERRLIST.
dnl Sets the cache variable ac_cv_var_sys_errlist to 'yes' or 'no'.
dnl
AC_DEFUN([AC_SYS_ERRLIST],
[AC_MSG_CHECKING(for sys_errlist)
AC_CACHE_VAL(ac_cv_var_sys_errlist,
[AC_TRY_LINK(,[extern char *sys_errlist[];
	sys_errlist[0];],
	ac_cv_var_sys_errlist=yes, ac_cv_var_sys_errlist=no)
])
AC_MSG_RESULT($ac_cv_var_sys_errlist)
if test $ac_cv_var_sys_errlist = yes; then
	AC_DEFINE(HAVE_SYS_ERRLIST, [], [Define the following if sys_errlist exists])
fi
])

dnl ##################################################################
dnl The macro AC_DECL_SYS_ERRLIST
dnl is based on code from configure.in distributed with LPRng-2.0.8.
dnl See the COPYING file for details.
dnl
dnl Usage:
dnl  AC_DECL_SYS_ERRLIST
dnl Check if sys_errlist is declared in one of the usual system headers.
dnl Tries to compile a program that uses sys_errlist.
dnl If the declaration exists, defines C preprocessor macro HAVE_SYS_ERRLIST_DECL.
dnl Sets the cache variable ac_cv_decl_sys_errlist to 'yes' or 'no'.
dnl
dnl This macro should be invoked in configure.in after all the header checks
dnl (e.g. AC_CHECK_HEADERS) have been performed, since our test program will
dnl #include "confdefs.h" (the C macro definitions accumulated by configure so far),
dnl then will #include various header files based on HAVE_STDLIB_H and HAVE_UNISTD_H we expect
dnl may appear in "confdefs.h".  Of course, to make this work, you should have invoked
dnl AC_CHECK_HEADERS(stdlib.h unistd.h) in configure.in.
dnl
AC_DEFUN([AC_DECL_SYS_ERRLIST],
[AC_MSG_CHECKING(for sys_errlist declaration)
AC_CACHE_VAL(ac_cv_decl_sys_errlist,
[AC_TRY_COMPILE([
#include "confdefs.h"    /* the header built by configure so far */
#include <stdio.h>
#include <errno.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif],
[printf("%s",sys_errlist[0]);],
ac_cv_decl_sys_errlist=yes, ac_cv_decl_sys_errlist=no)
])
AC_MSG_RESULT($ac_cv_decl_sys_errlist)
if test $ac_cv_decl_sys_errlist = yes; then
    AC_DEFINE(HAVE_SYS_ERRLIST_DECL, [], [Define the following if sys_nerr is declared])
fi
])


dnl ##################################################################
dnl The macro IST_SYS_SOCKET_IOCTLS_USE_STREAMS
dnl is by Irwin Tillman, and may be distributed under the GNU Public License.
dnl See the COPYING file for details.
dnl
dnl Version $Id: aclocal.m4,v 1.4 2008/10/17 20:09:12 root Exp $
dnl
dnl Usage:
dnl    IST_SYS_SOCKET_IOCTLS_USE_STREAMS
dnl Defines SYS_SOCKET_IOCTLS_USE_STREAMS if this system requires you to use STREAMS ioctls on sockets 
dnl instead of SIOCxxx ioctls on sockets.
dnl Also sets the cache variable ist_cv_sys_socket_ioctls_use_streams to 'yes' or 'no'.
dnl (Side effects: also defines cache variable ac_cv_header_stropts_h. and C preprocessor macro 
dnl HAVE_STROPTS_H based on whether stropts.h header is present.)
dnl
dnl Our approach is to guess that if your systems has stropts.h header (STREAMS user options definitions) 
dnl and is on a short list of known host platforms, that we should use STREAMS ioctls.  
dnl (Having a list of host platforms here isn't good; we really should try compiling and running a 
dnl test program instead.)
dnl
AC_DEFUN([IST_SYS_SOCKET_IOCTLS_USE_STREAMS], [
	dnl Check for stropts.h just for the side-effect of setting ac_cv_header_stropts_h.
	AC_CHECK_HEADERS(stropts.h)

	dnl We need the $host variable to have been set
	AC_REQUIRE([AC_CANONICAL_HOST])

	AC_CACHE_CHECK(if socket ioctls should use STREAMS instead of SIOCxxx, ist_cv_sys_socket_ioctls_use_streams, [
		if test "$ac_cv_header_stropts_h" = "yes"; then
			dnl stropts.h present, check for particular platform flavors
			case "$host" in
				*-sni-sysv*)	ist_cv_sys_socket_ioctls_use_streams=yes ;;
				*-*-ptx*)		ist_cv_sys_socket_ioctls_use_streams=yes ;;
	    		*) 				ist_cv_sys_socket_ioctls_use_streams=no  ;;
			esac
		else
			dnl stropts.h missing, assume we don't need to use stream ioctls
			ist_cv_sys_socket_ioctls_use_streams=no 
		fi
	])

	if test "$ist_cv_sys_socket_ioctls_use_streams" = "yes"; then
 		AC_DEFINE(SYS_SOCKET_IOCTLS_USE_STREAMS, [], [Define if STREAMS ioctls should be used on socket descriptors, instead of SIOCxxx ioctls])
	fi
])


dnl ##################################################################
dnl The macro IST_STRUCT_ORTENTRY
dnl is by Irwin Tillman, and may be distributed under the GNU Public License.
dnl See the COPYING file for details.
dnl
dnl Version $Id: aclocal.m4,v 1.4 2008/10/17 20:09:12 root Exp $
dnl
dnl Usage:
dnl    IST_STRUCT_ORTENTRY
dnl Defines HAVE_STRUCT_ORTENTRY if this system defines a struct ortentry.
dnl Also sets the cache variable ist_cv_struct_ortentry to 'yes' or 'no'.
dnl
dnl Side effects:
dnl Sets the cache variable ac_cv_header_sys_types_h to 'yes' or 'no', 
dnl    and C preprocessor macro HAVE_SYS_TYPES_H based on whether sys/types.h is present.
dnl Sets the cache variable ac_cv_header_sys_socket_h to 'yes' or 'no', 
dnl    and C preprocessor macro HAVE_SYS_SOCKET_H based on whether sys/socket.h is present.
dnl Sets the cache variable ac_cv_header_net_route_h to 'yes' or 'no', 
dnl    and C preprocessor macro HAVE_NET_ROUTE_H based on whether net/route.h is present.
dnl
dnl To improve the appearance of the resulting config.h.in file, you'll probably want to add 
dnl something like the following to your acconfig.h file: 
dnl   /* Define if struct ortentry is defined */
dnl   #undef  HAVE_STRUCT_ORTENTRY
dnl
AC_DEFUN([IST_STRUCT_ORTENTRY], [
	AC_CHECK_HEADERS(sys/types.h sys/socket.h net/route.h)

	AC_CACHE_CHECK(for ortentry{}, ist_cv_struct_ortentry, [

		AC_TRY_COMPILE(
			[
#				include "confdefs.h"  /* the header built by configure so far */
#               ifdef  HAVE_SYS_TYPES_H
#					include <sys/types.h>
#               endif
#               ifdef  HAVE_SYS_SOCKET_H
#					include <sys/socket.h>
#				endif
#				ifdef HAVE_NET_ROUTE_H
#					include <net/route.h>
#				endif
			],
			[struct ortentry foo],
			ist_cv_struct_ortentry=yes,
			ist_cv_struct_ortentry=no)
	])

	if test "$ist_cv_struct_ortentry" = "yes"; then
 		AC_DEFINE(HAVE_STRUCT_ORTENTRY, [], [Define if struct ortentry is defined])
	fi
])
